/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <iostream>
#include "equation.h"
#include "pptools.h"

int tabulation::setTables(double Xt[],double Yt[],short int nb_elem) {
	short int i;
	
	N_ELEM=nb_elem;
	if(nb_elem>MAX_TAB_N_PTS) return EXIT_FAILURE;
	for(i=0;i<N_ELEM;i++) {
		X[i]=Xt[i];
		Y[i]=Yt[i];
	}

	return EXIT_SUCCESS;
}

int tabulation::getValue(double x, unsigned short int interp_type) {
/* returns EXIT_SUCCESS if ok, 2 if out of table interpolation, EXIT_FAILURE if something went wrong*/
	int i,status,OutOfTable;
	double A,B;
	
	OutOfTable=EXIT_FAILURE;
	
	/* locating the two points used for calculating the interpolation curve */
	status=EXIT_FAILURE;
	i=0;
	while(x>X[i] && i<(N_ELEM-1)) i++;
	i--; //we want x to be between a smaller and a larger value
	if(i<0) { //if x is smaller than the first tabulated value, we will use the curve calculated from the first two points
		i=0;
		OutOfTable=2;
	}
	if(i>=N_ELEM-2) OutOfTable=2; //out of table, value too big
	
	/* calculating the interpolation */
	if(interp_type==InterpLinear) { //linear plot
		A=(Y[i+1]-Y[i])/(X[i+1]-X[i]);
		B=Y[i]-A*X[i];
		value=A*x+B;
		status=EXIT_SUCCESS;
	}
	if(interp_type==InterpXLog) { //X is log
		A=( Y[i+1]-Y[i] )/(log(X[i+1])-log(X[i]));
		B=Y[i]-A*log(X[i]);
		value=A*log(x)+B;
		status=EXIT_SUCCESS;
	}
	if(interp_type==InterpYLog) { //Y is log
		A=( log(Y[i+1])-log(Y[i]) )/( X[i+1]-X[i] );
		B=log(Y[i])-A*X[i];
		value=exp(A*x+B);
		status=EXIT_SUCCESS;
	}
	if(interp_type==InterpXYLog) { //X and Y are logs
		A=( log(Y[i+1])-log(Y[i]) )/(log(X[i+1])-log(X[i]));
		B=log(Y[i])-A*log(X[i]);
		value=exp(A*log(x)+B);
		status=EXIT_SUCCESS;
	}
	if(OutOfTable==2 && status==EXIT_SUCCESS) status=OutOfTable;
	return status;
}

double equation::df(int index_deri)
{
	double X;
	double F1,F2;
	
	X=parameters[index_deri];
	if(X==0) parameters[index_deri]=(-DeltaX/2);
		else parameters[index_deri]=X*(1-DeltaX/2);
	F1=f();
	if(X==0) parameters[index_deri]=(DeltaX/2);
		else parameters[index_deri]=X*(1+DeltaX/2);
	F2=f();
	parameters[index_deri]=X;
	
	if(X==0) return (F2-F1)/(DeltaX);
		else return (F2-F1)/(DeltaX*X);
}

int equation::Newton(int index_to_solve,double guess)
{
	double Xcur,Xnext;
	int i=0;
	
	Xnext=guess;
	do {
		message="";
		Xcur=Xnext;
		parameters[index_to_solve]=Xcur;
		Xnext=Xcur-f()/df(index_to_solve);
		i++;
	}
	while((fabs((Xnext-Xcur)/Xnext) > TolConv) && (i<MAX_EQ_ITERATIONS));
	parameters[index_to_solve]=Xcur;
	
	if(i<MAX_EQ_ITERATIONS) return EXIT_SUCCESS;
		else return EXIT_FAILURE;
}

int equation::solve(int index_to_solve,double guess,SolveType solveflags)
{
 /* because Newton's method doesn't know that results can not be <0, it will work only if it only has to
    increase the values compared to the initial guess. So, we provide a small guess, and if the result is not physically
    possible, we decrease that initial guess and try again.
    */
	double Xinit;
	const double Xfactor=1000;
	short int iteration=0;
	const short int max_iterations=4;
	
	Xinit=guess;
	if(solveflags==Positive) {	
    		do {
    			Newton(index_to_solve,Xinit);
			Xinit=Xinit/Xfactor;
			iteration++;
    		} while((parameters[index_to_solve]<0) && (iteration<=max_iterations));
	}
	if(solveflags==Negative) {	
    		do {
    			Newton(index_to_solve,Xinit);
			Xinit=Xinit/Xfactor;
			iteration++;
    		} while((parameters[index_to_solve]>0) && (iteration<=max_iterations));
	}
	if(solveflags==Any) {
		Newton(index_to_solve,Xinit);
	}
	
	return EXIT_SUCCESS;
}

double equation::f(void)
{
	printf("using f(x) from EQUATION class... this is not normal !\n");
	return EXIT_FAILURE;
}

