/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "inductance.h"

int inductance::setType(int choice) {
	
	switch (choice) {
		case 0: /* // wires */
			type=ParalleleWires;
			break;
		case 1: /* coaxial line */
			type=CoaxialLine;
			break;
		case 2: /* strip line */
			type=StripLine;
			break;
		case 3: /* converging line */
			type=ConvergingLine;
			break;
		case 4: /* disks */
			type=Disks;
			break;
		case 5: /* rectangular coil */
			type=RectangularCoil;
			break;
		case 6: /* helix */
			type=Helix;
			break;
		default:
			return EXIT_FAILURE;
	}
	return EXIT_SUCCESS;

}

double inductance::Ksh(double a) {
/* Calculates the coefficient for magnetic field leakage in a strip line */
	tabulation tab_Ksh;
	double K;
	int status;
	const short int Npts=25;
	
	double X[Npts] = { 
	0.092, 0.107, 0.120, 0.139, 0.165, 0.192, 0.223, 0.253, 0.320, 0.367, 0.446, 0.517, 0.641, 0.749, 0.894, 1.130, 1.429, 1.897, 2.376, 3.155, 4.315, 5.511, 7.319, 8.644, 9.533 
	};
	double Y[Npts] = {
	0.876, 0.863, 0.851, 0.835, 0.813, 0.793, 0.772, 0.752, 0.715, 0.691, 0.656, 0.629, 0.584, 0.551, 0.514, 0.465, 0.413, 0.356, 0.313, 0.263, 0.214, 0.180, 0.147, 0.132, 0.124
	};
	status=tab_Ksh.setTables(X,Y,Npts);
	status=tab_Ksh.getValue(a,tabulation::InterpXLog);
	K=tab_Ksh.value;
	if(K>1.0) K=1.0;
	if(K<0.0) K=0.0;
	message=QString("Strip line field leakage correction factor: %1").arg(K,0,'g',2);
	if(status==EXIT_FAILURE) message="Error, calculation is wrong !";
	if(status==2) message.append("\nOut of table interpolation");
	printf("%s \n",message.ascii());
	
	return K;
}

double inductance::Kl(double a) {
/* Calculates the coefficient for magnetic field leakage in a strip line */
	tabulation tab_Kl;
	double K;
	int status;
	const short int Npts=28;
	
	double X[Npts] = { 
	0.010, 0.018, 0.028, 0.046, 0.070, 0.093, 0.115, 0.144, 0.186, 0.262, 0.339, 0.446, 0.616, 0.865, 1.234, 1.572, 2.137, 3.201, 4.794, 6.208, 9.150, 13.484, 16.369, 21.544, 26.580, 40.459, 60.598, 93.741
	};
	double Y[Npts] = {
	0.043, 0.062, 0.085, 0.120, 0.165, 0.204, 0.239, 0.275, 0.323, 0.389, 0.441, 0.511, 0.586, 0.669, 0.744, 0.788, 0.836, 0.885, 0.920, 0.938, 0.956, 0.974, 0.979, 0.988, 0.993, 0.994, 0.999, 1.004
	};
	
	status=tab_Kl.setTables(X,Y,Npts);
	status=tab_Kl.getValue(a,tabulation::InterpXLog);
	K=tab_Kl.value;
	if(K>1.0) K=1.0;
	if(K<0.0) K=0.0;
	message=QString("Solenoid field leakage correction factor: %1").arg(K,0,'g',2);
	if(status==EXIT_FAILURE) message="Error, calculation is wrong !";
	if(status==2) message.append("\nOut of table interpolation");
	printf("%s \n",message.ascii());
	
	return K;
}

double inductance::Ns(double a) {
/* Calculates the coefficient for magnetic field leakage in a strip line */
	tabulation tab_N;
	double K;
	int status;
	const short int Npts=32;
	
	double X[Npts] = { 
	1.011, 1.095, 1.198, 1.324, 1.463, 1.633, 1.822, 1.974, 2.159, 2.385, 2.609, 2.854, 3.153, 3.733, 4.164, 4.785, 5.609, 6.573, 7.856, 9.205, 10.364, 13.011, 15.855, 19.318, 23.075, 27.022, 33.239, 40.890, 54.422, 71.009, 83.129, 101.239
	};
	double Y[Npts] = {
	0.008, 0.020, 0.034, 0.052, 0.068, 0.083, 0.098, 0.110, 0.122, 0.133, 0.145, 0.157, 0.169, 0.186, 0.197, 0.210, 0.224, 0.236, 0.248, 0.258, 0.264, 0.275, 0.284, 0.292, 0.298, 0.302, 0.307, 0.312, 0.317, 0.321, 0.322, 0.325
	};
	
	status=tab_N.setTables(X,Y,Npts);
	status=tab_N.getValue(a,tabulation::InterpXLog);
	K=tab_N.value;
	if(K>0.325) K=0.325;
	if(K<0.0) K=0.0;
	message=QString("Helix N* correction factor: %1").arg(K,0,'g',2);
	if(status==EXIT_FAILURE) message="Error, calculation is wrong !";
	if(status==2) message.append("\nOut of table interpolation");
	printf("%s \n",message.ascii());
	
	return K;
}

double inductance::Gs(double a) {
/* Calculates the coefficient for magnetic field leakage in a strip line */
	tabulation tab_G;
	double K;
	int status;
	const short int Npts=19;
	
	double X[Npts] = { 
	0.029, 0.033, 0.037, 0.046, 0.054, 0.064, 0.077, 0.095, 0.121, 0.154, 0.201, 0.267, 0.326, 0.405, 0.498, 0.595, 0.697, 0.832, 0.994
	};
	double Y[Npts] = {
	-2.969, -2.836, -2.694, -2.514, -2.343, -2.173, -2.002, -1.794, -1.567, -1.339, -1.074, -0.799, -0.600, -0.392, -0.183, -0.003, 0.158, 0.328, 0.508
	};
	
	status=tab_G.setTables(X,Y,Npts);
	status=tab_G.getValue(a,tabulation::InterpXLog);
	K=tab_G.value;
	if(K>0.51) K=0.51;
	if(K<-3) K=-3;
	message=QString("Helix G* correction factor: %1").arg(K,0,'g',2);
	if(status==EXIT_FAILURE) message="Error, calculation is wrong !";
	if(status==2) message.append("\nOut of table interpolation");
	printf("%s \n",message.ascii());
	
	return K;
}

double inductance::parallele_wires() {
	double length,gap,a,p;
	
	a=parameters[1];
	gap=parameters[2];
	length=parameters[3];
		
	p=sqrt(0.25*gap*gap-a*a);
	L=-MU_0/PI*length*(log(gap/a)+log(0.5-p/gap));
	
	return L;
}

double inductance::coaxial_line() {
	double length,r1,r2;
	
	r1=parameters[1];
	r2=parameters[2];
	length=parameters[3];
	
	L=MU_0/(2.0*PI)*length*log(r2/r1);

	return L;
}

double inductance::strip_line() {
	double a,gap,width,length;
	
	width=parameters[1];
	gap=parameters[2];
	length=parameters[3];
	a=gap/width;
	
	L=MU_0*gap*length/width*Ksh(a);
	return L;
}

double inductance::converging_line() {
	double a,gap,w1,w2,length;
	double Kco;
	
	w2=parameters[1];
	gap=parameters[2];
	length=parameters[3];
	w1=parameters[4];
	a=gap/((w1+w2)/2.0);
	Kco=Ksh(a);
	
	L=MU_0*gap*length/(w1-w2)*log(w1/w2)*Kco;
	return L;
}

double inductance::disks() {
	double r1,r2,g1,g2;
	
	r1=parameters[1];
	r2=parameters[2];
	g1=parameters[3];
	g2=parameters[4];
	
	L=MU_0/(2.0*PI)*(g2-g1+ (g1*r2-g2*r1)/(r2-r1)*log(r2/r1) );
	return L;
}

double inductance::rectangular_coil() {
	double width,gap,length;
	double regime,c,Kr=-1;
	
	width=parameters[1];
	gap=parameters[2];
	length=parameters[3];
	
	regime=gap/length;
	if(regime<=0.2) Kr=Ksh(gap/width);
	if(regime>=0.3) Kr=Kl( width/(2*sqrt((gap*length)/PI)) );
	if(regime>0.2 && regime<0.3) {
	    c=(regime-0.2)/(0.3-0.2);
	    Kr=c*Kl( width/(2*sqrt((gap*length)/PI)) )+(1-c)*Ksh(gap/width);
	    message.append("\nWarning: between two models. Please, check results!");
	}
	L=MU_0*length*gap/width*Kr;
	
	return L;
}

double inductance::helix() {
	double radius,height,wire_radius,pitch;
	double Ls,kl,gs,ns,S,N;
	
	radius=parameters[1];
	height=parameters[2];
	wire_radius=parameters[3];
	pitch=parameters[4];
	
	S=PI*radius*radius;
	N=height/pitch;
	
	kl=Kl(height/(2*radius));
	Ls=MU_0*S/height*kl;
	gs=Gs(2*radius/pitch);
	ns=Ns(N);
	
	L=Ls-MU_0*radius*N*(gs+ns);
	
	return L;
}

double inductance::L2C() {
	L=parameters[5];
	
	if(L>0) {
		C=E_0*MU_0/L;
	} else C=0;
	parameters[6]=C;
	
	return C;
}

double inductance::C2L() {
	C=parameters[6];
	
	if(C>0) {
		L=E_0*MU_0/C;
	} else L=0;
	parameters[5]=L;
	parameters[0]=L;
	
	return L;
}

double inductance::f() {
//this function is a wrapper needed by the solver
	switch(type) {
		case ParalleleWires:
			return parameters[0]-parallele_wires();
			break;
		case CoaxialLine:
			return parameters[0]-coaxial_line();
			break;
		case StripLine:
			return parameters[0]-strip_line();
			break;
		case ConvergingLine:
			return parameters[0]-converging_line();
			break;
		case Disks:
			return parameters[0]-disks();
			break;
		case RectangularCoil:
			return parameters[0]-rectangular_coil();
			break;
		case Helix:
			return parameters[0]-helix();
			break;
		default:
			return EXIT_FAILURE;	
	}
}

void inductance::CalculateLandC() {
//we know that L and C have to be calculated -> direct evaluation
	parameters[0]=0;
	L=-1*f();
	parameters[0]=L;
	parameters[5]=L;
	parameters[6]=L2C();
}
