/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <iostream>
#include <qstring.h>
#include "liquidresistor.h"
#include "pptools.h"

liquidResistor::liquidResistor()
{
	nb_params=3;
	message="";
}

void liquidResistor::setSolution(QString choice)
{
	const short int Npts=7;
	double Xcuso4[Npts] = {
	0.0005, 0.001, 0.005, 0.01, 0.02, 0.05, 0.1
	};
	double Ycuso4[Npts] = {
	121.6, 115.20, 94.02, 83.08, 72.16, 59.02, 50.55
	};
	double Ynacl[Npts] = {
	124.44, 123.68, 120.59, 118.45, 115.70, 111.01, 106.69
	};
	const short int NptsKmno4=4;
	double Xkmno4[NptsKmno4] = {
	0.0005, 0.001, 0.01, 0.1
	};
	double Ykmno4[NptsKmno4] = {
	132.7, 131.9, 126.5, 113
	};
	
	if(choice=="KMnO4") { //potassium permanganate
		Amol=39.0983+54.93805+4*15.9994;
		Lambda0=134.8;
		tab_resistivity.setTables(Xkmno4,Ykmno4,NptsKmno4);
	}
	if(choice=="CuSO4") { //copper sulfate
		Amol=63.546+32.066+4*15.9994;
		Lambda0=133.6;
		tab_resistivity.setTables(Xcuso4,Ycuso4,Npts);
	}
	if(choice=="NaCl") { //salt
		Amol=22.989768+35.4527;
		Lambda0=126.39;
		tab_resistivity.setTables(Xcuso4,Ynacl,Npts);
	}
	
}

double liquidResistor::f(void)
{
	int status;
	double R,concentration,length,diameter,rho;
	
	R=parameters[0];
	concentration=parameters[1];
	length=parameters[2];
	diameter=parameters[3]*1.0e-3;
	
	if((concentration/Amol)>0.0005) {
		status=tab_resistivity.getValue(concentration/Amol,tabulation::InterpXLog);
		if(status==EXIT_FAILURE) message="Error, calculation is wrong !";
		if(status==2) message.append("\nOut of table interpolation");
		rho=tab_resistivity.value;
	} else {
		rho=Lambda0-((60.20+0.229*Lambda0)*pow(concentration/Amol,0.5));
		message.append("\nOut of table interpolation, using simplified model");
	}
	
	return R-rho*length/(PI*diameter*diameter/4);
}

