/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include <qstring.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include "pptools.h"
#include "rlc.h"
#include "equation.h"
#include "inductance.h"
#include "liquidresistor.h"
#include "skindepth.h"
#include "colorcoding.h"
#include "help.h"

int RLCwindow::solve_rlc()
{
    rlc circuit;
    int index_to_solve[2];
    int index=0;
    QString buffer;
    bool ok;
        
    buffer=rlc_input_1->text();
    if(buffer.isEmpty()) index_to_solve[index++]=1;
    else {
	circuit.parameters[1]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    text_status->setText("Wrong format for tmax");
	    return EXIT_FAILURE;
	}
    }
    buffer=rlc_input_2->text();
    if(buffer.isEmpty()) index_to_solve[index++]=2;
    else {
	circuit.parameters[2]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    text_status->setText("Wrong format for imax");
	    return EXIT_FAILURE;
	}
    }
    buffer=rlc_input_3->text();
    if(buffer.isEmpty()) index_to_solve[index++]=3;
    else {
	circuit.parameters[3]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    text_status->setText("Wrong format for U0");
	    return EXIT_FAILURE;
	}
    }
    buffer=rlc_input_4->text();
    if(buffer.isEmpty()) index_to_solve[index++]=4;
    else {
	circuit.parameters[4]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    text_status->setText("Wrong format for R");
	    return EXIT_FAILURE;
	}
    }
    buffer=rlc_input_5->text();
    if(buffer.isEmpty()) index_to_solve[index++]=5;
    else {
	circuit.parameters[5]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    text_status->setText("Wrong format for L");
	    return EXIT_FAILURE;
	}
    }
    buffer=rlc_input_6->text();
    if(buffer.isEmpty()) index_to_solve[index++]=6;
    else {
	circuit.parameters[6]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    text_status->setText("Wrong format for C");
	    return EXIT_FAILURE;
	}
    }
   
    circuit.solve(index_to_solve);
    text_status->setText(circuit.message);
    
    rlc_input_1->setText(buffer.setNum(circuit.parameters[1],'g',PRECISION));
    rlc_input_2->setText(buffer.setNum(circuit.parameters[2],'g',PRECISION));
    rlc_input_3->setText(buffer.setNum(circuit.parameters[3],'g',PRECISION));
    rlc_input_4->setText(buffer.setNum(circuit.parameters[4],'g',PRECISION));
    rlc_input_5->setText(buffer.setNum(circuit.parameters[5],'g',PRECISION));
    rlc_input_6->setText(buffer.setNum(circuit.parameters[6],'g',PRECISION));
    
    return EXIT_SUCCESS;
}


int RLCwindow::solve_liquid_R()
{
    QString buffer;
    bool ok;
    int index_to_solve=0;
    liquidResistor lR;
    
    buffer=diameter_input->text();
    if(buffer.isEmpty()) index_to_solve=3;
    else {
	lR.parameters[3]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    return EXIT_FAILURE;
	}
    }
    buffer=resistance_input->text();
    if(buffer.isEmpty()) index_to_solve=0;
    else {
	lR.parameters[0]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    return EXIT_FAILURE;
	}
    }
    buffer=length_input->text();
    if(buffer.isEmpty()) index_to_solve=2;
    else {
	lR.parameters[2]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    return EXIT_FAILURE;
	}
    }
    buffer=concentration_input->text();
    if(buffer.isEmpty()) index_to_solve=1;
    else {
	lR.parameters[1]=buffer.toDouble(&ok);
	if(ok==FALSE) {
	    return EXIT_FAILURE;
	}
    }
    
    lR.setSolution(solution_select->currentText());
    lR.solve(index_to_solve,10,equation::Positive);
    printf("%s\n",lR.message.ascii());
    
    diameter_input->setText(buffer.setNum(lR.parameters[3],'g',PRECISION));
    length_input->setText(buffer.setNum(lR.parameters[2],'g',PRECISION));
    resistance_input->setText(buffer.setNum(lR.parameters[0],'g',PRECISION));
    concentration_input->setText(buffer.setNum(lR.parameters[1],'g',PRECISION));
    
    return EXIT_SUCCESS;
}


int RLCwindow::solve_color_coding()
{
	int index_to_solve=0;
	QString buffer;
	bool ok;
	colorCoding code;
	
	code.parameters[1]=(float)band1_input->currentItem();
	code.parameters[2]=(float)band2_input->currentItem();
	code.parameters[3]=(float)band3_input->currentItem();
	buffer=color_resistance_input->text();
	if(buffer.isEmpty()) index_to_solve=0;
	else {
		index_to_solve=1;
		code.parameters[0]=buffer.toDouble(&ok);
	}
	
	code.solve(index_to_solve,0,equation::Positive);
	
	band1_input->setCurrentItem((int)code.parameters[1]);
	band2_input->setCurrentItem((int)code.parameters[2]);
	band3_input->setCurrentItem((int)code.parameters[3]);
	color_resistance_input->setText(buffer.setNum(code.parameters[0],'g',PRECISION));
	
	return EXIT_SUCCESS;
}


int RLCwindow::solve_inductance()
{
	QString buffer;
	bool ok;
	inductance indu;
	int index, index_to_solve=0;
	const short int max_fields=6;
	QLineEdit *input_table[max_fields] = {L_input_1, L_input_2, L_input_3, L_input_4, L_input_5, L_input_6};
	int type_inductance;
	/* this is not very robust: it would be MUCH better to start in position 0 with the widget matching parameters[0],
	then the other one, and then at the end of the list, the optional widgets.
	So the table could start at index0=parameters0 and we would stop reading values when reaching the last used input field
	-> to do for a next release... */
	
	//zeroing the parameters
	type_inductance=inductance_type_select->currentItem();
	for(index=0;index<=max_fields;index++) {
		indu.parameters[index]=0;
	}
	//reading the values
	for(index=0; index<max_fields; index++){
		buffer=input_table[index]->text();
		if(buffer.isEmpty()) {
			if(index_to_solve==0) index_to_solve=index+1;
		}
		else {
			indu.parameters[index+1]=buffer.toDouble(&ok);
			if(ok==FALSE) {
				printf("Parameter %d invalid\n",index+1);
				return EXIT_FAILURE;
			}
		}
	}
	buffer=input_table[4]->text();
	if(buffer.isEmpty()) {
		indu.C2L(); //we need to calculate the L matching the given C
	}
	indu.parameters[0]=indu.parameters[5];
	if((index_to_solve==5) || (index_to_solve==6)) index_to_solve=0;
	
	//setting the selecting inductance geometry
	indu.setType(inductance_type_select->currentItem());
	
	//calculating the needed value
	if(index_to_solve==0) indu.CalculateLandC();
	else {
		indu.solve(index_to_solve,1e-3,equation::Positive);
		indu.L2C();
	}
	
	//displaying the results
	for(index=0; index<max_fields; index++){
		input_table[index]->setText(buffer.setNum(indu.parameters[index+1],'g',PRECISION));
	}
	L_status_text->setText(indu.message);
	
	return EXIT_SUCCESS;
}


int RLCwindow::refresh_inductance()
{
/* 	0 = parallel wires
	1 = coax
	2 = strip line
	3 = converging strip line
	4 = disks
	5 = rectangular coil
	6 = helix
 */
 	const short int max_fields=4;
	const short int nb_types=7;
	int i_field;
	int type_inductance;
	int max_field[nb_types]={3,3,3,4,4,3,4};
	
	QLabel *label_table[max_fields][2] = { {L_label_1,L_unit_1} , {L_label_2,L_unit_2} , {L_label_3,L_unit_3} , {L_label_4,L_unit_4} };
	QLineEdit *input_table[max_fields] = {L_input_1, L_input_2, L_input_3, L_input_4};
	
	QString labels[nb_types][max_fields][3]={
	{ {"wire radius:", "radius of the wire", "m"}, {"gap:", "gap between the two wires", "m"}, {"length:", "length of the wires", "m"}, {"", "", ""} },
	{ {"inside radius:", "inside conductor radius", "m"}, {"outside radius:", "outside conductor radius", "m"}, {"length:", "length of the coaxial line", "m"}, {"", "", ""} },
	{ {"width:", "width of the strip line", "m"}, {"gap:", "gap between the two electrodes", "m"}, {"length:", "length of the strip line", "m"}, {"", "", ""} },
	{ {"small width:", "width of the smallest side of the strip line", "m"}, {"gap:", "gap between the two electrodes", "m"}, {"length:", "length of the strip line", "m"}, {"large width:", "width of the largest side of the strip line", "m"} },
	{ {"small radius 1:", "smallest radius of the circular electrodes", "m"}, {"large radius 2:", "largest radius of the circular electrodes", "m"}, {"gap 1:", "gap at the smallest radius", "m"}, {"gap 2", "gap at the largest radius", "m"} },
	{ {"width:", "width of the coil", "m"}, {"height:", "height of the coil", "m"}, {"length:", "length of the coil", "m"}, {"", "", ""} },
	{ {"radius:", "radius of the helix", "m"}, {"height:", "height of the helix", "m"}, {"wire radius:", "radius of the wire making the coil", "m"}, {"pitch:", "pitch of the helix (length between two turns)", "m"} },
	};
	
	type_inductance=inductance_type_select->currentItem();
	
	// filling out the fields labels, units, tooltips
	for(i_field=0;i_field<max_field[type_inductance];i_field++) {
		//setting the text
		label_table[i_field][0]->setText(labels[type_inductance][i_field][0]);
		QToolTip::add(input_table[i_field],labels[type_inductance][i_field][1]);
		label_table[i_field][1]->setText(labels[type_inductance][i_field][2]);
		//showing the widget (in case it has been hidden before)
		label_table[i_field][0]->show();
		input_table[i_field]->show();
		label_table[i_field][1]->show();
	}
	// hidding unecessary fields
	for(i_field=max_field[type_inductance];i_field<max_fields;i_field++) {
		label_table[i_field][0]->hide();
		input_table[i_field]->hide();
		label_table[i_field][1]->hide();
	}
	return EXIT_SUCCESS;	
}


void RLCwindow::solve_skin_depth()
{
	skinDepth sk;
	short int index_to_solve=0;
	QString buffer;
	bool ok;
	
	buffer=skin_depth_input->text();
	if(buffer.isEmpty()) index_to_solve=0;
		else sk.parameters[0]=buffer.toDouble(&ok);
	buffer=risetime_input->text();
	if(buffer.isEmpty()) index_to_solve=1;
		else sk.parameters[1]=buffer.toDouble(&ok);
	buffer=current_time_input->text();
	if(buffer.isEmpty()) index_to_solve=2;
		else sk.parameters[2]=buffer.toDouble(&ok);
	
	sk.setParameters((skinDepth::TypeShape)select_waveform->currentItem(),select_material->currentText());
	sk.solve(index_to_solve,1e-9,equation::Positive);
	
	text_skin_depth->setText(sk.message);
	
	skin_depth_input->setText(buffer.setNum(sk.parameters[0],'g',PRECISION));
	risetime_input->setText(buffer.setNum(sk.parameters[1],'g',PRECISION));
	current_time_input->setText(buffer.setNum(sk.parameters[2],'g',PRECISION));
}


void RLCwindow::HelpOpen()
{
    Help dlg( this, 0, TRUE );
    dlg.exec();
}
